#if HAVE_CONFIG_H
#include "framecpp/config.h"
#endif /* HAVE_CONFIG_H */

#include <string.h>
#include <unistd.h>

#include <iostream>
#include <list>
#include <memory>

#include "general/AtExit.hh"
#include "general/types.hh"

#if 0
#include "framecpp/Common/FrameSpec.hh"
#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/FrameStream.hh"
#include "framecpp/Common/FrTOC.hh"
#endif /* 0 */

#include "framecpp/IFrameStream.hh"
#include "framecpp/FrameCPP.hh"
#include "framecpp/FrAdcData.hh"
#include "framecpp/FrVect.hh"

using std::filebuf;
using namespace FrameCPP;

using FrameCPP::Common::FrameBuffer;

char* programname = 0;

typedef std::list< const char* > channel_set_type;
channel_set_type	ChannelSet;

void dump_vect( const FrVect* Vect );

template< class T > void
dump( const CHAR_U* Source, INT_4U NData );

template< > void
dump< CHAR >( const CHAR_U* Source, INT_4U NData )
{
  const CHAR* data( reinterpret_cast< const CHAR* >( Source ) );

  for ( INT_4U x = 0;
	x < NData;
	++x, ++data )
  {
    if ( x )
    {
      std::cout << ", ";
    }
    std::cout << (INT_2S)( *data );
  }
}

template< > void
dump< CHAR_U >( const CHAR_U* Source, INT_4U NData )
{
  const CHAR_U* data( reinterpret_cast< const CHAR_U* >( Source ) );

  for ( INT_4U x = 0;
	x < NData;
	++x, ++data )
  {
    if ( x )
    {
      std::cout << ", ";
    }
    std::cout << (INT_2U)( *data );
  }
}

template< class T > void
dump( const CHAR_U* Source, INT_4U NData )
{
  const T* data( reinterpret_cast< const T* >( Source ) );

  for ( INT_4U x = 0;
	x < NData;
	++x, ++data )
  {
    if ( x )
    {
      std::cout << ", ";
    }
    std::cout << *data;
  }
}

int
main(int argc, char* argv[])
try
{
  /* Determine program name (strip filesystem path) */
  for (programname = argv [0] + strlen (argv [0]) - 1;
       programname != argv [0] && *programname != '/';
       programname--)
    ;
  
  if (*programname == '/')
  {
    programname++;
  }


  //---------------------------------------------------------------------
  // Parse arguments
  //---------------------------------------------------------------------
  int		args_parsed;
  
  for ( args_parsed = 1; args_parsed < argc; ++args_parsed )
  {
    char* cur_arg = argv[ args_parsed ];
	
    if ( ( *cur_arg == '-' ) && ( strlen( cur_arg ) > 2 ) )
    {
      if ( strncmp( cur_arg, "--channel=", 10 ) == 0 )
      {
	ChannelSet.push_back( &cur_arg[ 10 ] );
      }
    }
    else
    {
      break;
    }
  }
  //---------------------------------------------------------------------
  // Process files
  //---------------------------------------------------------------------
  for ( ; args_parsed < argc; ++args_parsed )
  {
    char* filename = argv[ args_parsed ];
    //-------------------------------------------------------------------
    // Creation of the frame structure by reading of frame file
    //-------------------------------------------------------------------
    FrameBuffer< filebuf >*
      ibuf( new FrameBuffer< filebuf >( std::ios::in ) );
    
    ibuf->open( filename, std::ios::in | std::ios::binary );

    IFrameStream	ifs( ibuf );

    std::cout << "Frame file: " << filename << std::endl
	      << "Frame Specification: " << ifs.Version( ) << std::endl
      ;
    for ( channel_set_type::const_iterator
	    cur = ChannelSet.begin( ),
	    last = ChannelSet.end( ); 
	  cur != last;
	  ++cur )
    {
      //-----------------------------------------------------------------
      // FrAdcData
      //-----------------------------------------------------------------
      std::auto_ptr< FrAdcData >	adc;

      adc.reset( ifs.ReadFrAdcData( 0, *cur ) );
      if ( adc.get( ) )
      {
	dump_vect( adc->RefData( )[ 0 ] );
	continue;
      }
    }

  }
  //---------------------------------------------------------------------
  // Exit
  //---------------------------------------------------------------------
  General::AtExit::Cleanup( );
}
catch( ... )
{
  General::AtExit::Cleanup( );
}

void
dump_vect( const FrVect* Vect )
{
  if ( Vect == (const FrVect*)NULL )
  {
    return;
  }
  std::cout
    << "Name:     " << Vect->GetName( ) << std::endl
    << "Compress: " << Vect->GetCompress( )
    << " (Endianness: " << ( ( Vect->GetCompress( ) & 0x100 )
			     ? "Little" : "Big" )
    << " Scheme: " << ( Vect->GetCompress( ) & 0xFF ) << ")"
    << std::endl
    << "Type:     " << Vect->GetType( ) << std::endl
    << "NData:    " << Vect->GetNData( ) << std::endl
    << "NBytes:   " << Vect->GetNBytes( ) << std::endl
    ;
  std::cout << "Data: ";
  General::AutoArray< CHAR_U >	expanded;
  switch ( Vect->GetType( ) )
  {
  case Version::FrVectDataTypes::FR_VECT_2U:
    dump< INT_2U >( Vect->GetDataUncompressed( expanded ), Vect->GetNData( ) );
    break;
  }
  std::cout << std::endl;
}
