/* -*- mode: c++; c-basic-offset: 2; -*- */

#include "general/config.h"

#include <iostream>
#include <stdexcept>

#include "general/Thread.hh"

static General::Thread::thread_type NULL_THREAD =
  (General::Thread::thread_type)0;

namespace General
{
  Thread::
  Thread( )
    : m_tid( NULL_THREAD ),
      m_key( MutexLock::Initialize( ) )
  {
  }

  Thread::
  ~Thread( )
  {
    Join( );
  }

  bool Thread::
  operator==( const Thread& Source )
  {
    return( Source.m_tid == m_tid );
  }

  void Thread::
  Cancel( ) const
  {
    if ( m_tid != NULL_THREAD )
    {
      pthread_cancel( m_tid );
    }
  }

  void Thread::
  Detach( ) const
  {
    if ( m_tid != NULL_THREAD )
    {
      // Detach the current thread (makes it non-joinable).
      volatile thread_type tmp = m_tid;

      {
	MutexLock	l( m_key, __FILE__, __LINE__ );

	m_tid = NULL_THREAD;
      }
      pthread_detach( tmp );
    }
  }

  bool Thread::
  IsAlive( ) const
  {
    if ( ( m_tid )
	 && ( pthread_kill( m_tid, 0 ) == 0 ) )
    {
      return true;
    }
    return false;
  }

  void Thread::
  Join( ) const
  {
    if ( m_tid != NULL_THREAD )
    {
      pthread_join( m_tid, NULL );
      {
	MutexLock	l( m_key, __FILE__, __LINE__ );
	m_tid = NULL_THREAD;
      }
    }
  }

  void Thread::
  Kill( int Signal ) const
  {
    if ( m_tid != NULL_THREAD )
    {
      pthread_kill( m_tid, Signal );
    }
  }

  void Thread::
  Start( )
  {
    if ( m_tid == NULL_THREAD )
    {

      int retval = pthread_create( &m_tid, NULL,
				   (void*(*)(void*))start_routine,
				   this );
      if ( retval != 0 )
      {
	throw std::runtime_error( "General::Thread::Start: failed to start thread" );
      }
    }
  }

  Thread* Thread::
  start_routine( Thread* ThreadSource )
  {
    ThreadSource->run( );

    return ThreadSource;
  }
}
